@@ -61,7 +61,7 @@ ldns_dname_cat_clone(const ldns_rdf *rd1, const ldns_rdf *rd2)
 	/* put the two dname's after each other */
 	memcpy(buf, ldns_rdf_data(rd1), left_size);
 	memcpy(buf + left_size, ldns_rdf_data(rd2), ldns_rdf_size(rd2));
-	
+
 	new = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_DNAME, new_size, buf);
 
 	LDNS_FREE(buf);
@@ -90,7 +90,7 @@ ldns_dname_cat(ldns_rdf *rd1, ldns_rdf *rd2)
 	size = left_size + ldns_rdf_size(rd2);
 
 	ldns_rdf_set_data(rd1, LDNS_XREALLOC(ldns_rdf_data(rd1), uint8_t, size));
-	memcpy(ldns_rdf_data(rd1) + left_size, ldns_rdf_data(rd2), 
+	memcpy(ldns_rdf_data(rd1) + left_size, ldns_rdf_data(rd2),
 			ldns_rdf_size(rd2));
 	ldns_rdf_set_size(rd1, size);
 
@@ -129,7 +129,7 @@ ldns_dname_clone_from(const ldns_rdf *d, uint16_t n)
 	uint8_t *data;
 	uint8_t label_size;
 	size_t data_size;
-	
+
 	if (!d ||
 	    ldns_rdf_get_type(d) != LDNS_RDF_TYPE_DNAME ||
 	    ldns_dname_label_count(d) < n) {
@@ -161,7 +161,7 @@ ldns_dname_left_chop(const ldns_rdf *d)
 	if (!d) {
 		return NULL;
 	}
-		
+
 	if (ldns_rdf_get_type(d) != LDNS_RDF_TYPE_DNAME) {
 		return NULL;
 	}
@@ -177,9 +177,9 @@ ldns_dname_left_chop(const ldns_rdf *d)
 	return chop;
 }
 
-uint8_t         
+uint8_t
 ldns_dname_label_count(const ldns_rdf *r)
-{       
+{
         uint16_t src_pos;
         uint16_t len;
         uint8_t i;
@@ -189,28 +189,28 @@ ldns_dname_label_count(const ldns_rdf *r)
 		return 0;
 	}
 
-        i = 0; 
+	i = 0;
 	src_pos = 0;
-        r_size = ldns_rdf_size(r);
+	r_size = ldns_rdf_size(r);
 
-        if (ldns_rdf_get_type(r) != LDNS_RDF_TYPE_DNAME) {
-                return 0;
-        } else {
-                len = ldns_rdf_data(r)[src_pos]; /* start of the label */
+	if (ldns_rdf_get_type(r) != LDNS_RDF_TYPE_DNAME) {
+		return 0;
+	} else {
+		len = ldns_rdf_data(r)[src_pos]; /* start of the label */
 
-                /* single root label */
-                if (1 == r_size) {
-                        return 0; 
-                } else {
-                        while ((len > 0) && src_pos < r_size) {
-                                src_pos++;
-                                src_pos += len;
-                                len = ldns_rdf_data(r)[src_pos];
-                                i++;
-                        }
-                }
-                return i;
-        }
+		/* single root label */
+		if (1 == r_size) {
+			return 0;
+		} else {
+			while ((len > 0) && src_pos < r_size) {
+				src_pos++;
+				src_pos += len;
+				len = ldns_rdf_data(r)[src_pos];
+				i++;
+			}
+		}
+	}
+	return i;
 }
 
 ldns_rdf *
@@ -262,8 +262,11 @@ ldns_dname_is_subdomain(const ldns_rdf *sub, const ldns_rdf *parent)
 	uint8_t sub_lab;
 	uint8_t par_lab;
 	int8_t i, j;
-	ldns_rdf *tmp_sub;
-	ldns_rdf *tmp_par;
+	ldns_rdf *tmp_sub = NULL;
+	ldns_rdf *tmp_par = NULL;
+    ldns_rdf *sub_clone;
+    ldns_rdf *parent_clone;
+    bool result = true;
 
 	if (ldns_rdf_get_type(sub) != LDNS_RDF_TYPE_DNAME ||
 			ldns_rdf_get_type(parent) != LDNS_RDF_TYPE_DNAME ||
@@ -271,39 +274,49 @@ ldns_dname_is_subdomain(const ldns_rdf *sub, const ldns_rdf *parent)
 		return false;
 	}
 
-	sub_lab = ldns_dname_label_count(sub);
-	par_lab = ldns_dname_label_count(parent);
+    /* would be nicer if we do not have to clone... */
+    sub_clone = ldns_dname_clone_from(sub, 0);
+    parent_clone = ldns_dname_clone_from(parent, 0);
+    ldns_dname2canonical(sub_clone);
+    ldns_dname2canonical(parent_clone);
+
+	sub_lab = ldns_dname_label_count(sub_clone);
+	par_lab = ldns_dname_label_count(parent_clone);
 
 	/* if sub sits above parent, it cannot be a child/sub domain */
 	if (sub_lab < par_lab) {
-		return false;
-	}
-	
-	/* check all labels the from the parent labels, from right to left. 
-	 * When they /all/ match we have found a subdomain
-	 */
-	j = sub_lab - 1; /* we count from zero, thank you */
-	for (i = par_lab -1; i >= 0; i--) {
-		tmp_sub = ldns_dname_label(sub, j);
-		tmp_par = ldns_dname_label(parent, i);
-		if (!tmp_sub || !tmp_par) {
-			/* deep free does null check */
-			ldns_rdf_deep_free(tmp_sub);
-			ldns_rdf_deep_free(tmp_par);
-			return false;
-		}
+		result = false;
+	} else {
+		/* check all labels the from the parent labels, from right to left.
+		 * When they /all/ match we have found a subdomain
+		 */
+		j = sub_lab - 1; /* we count from zero, thank you */
+		for (i = par_lab -1; i >= 0; i--) {
+			tmp_sub = ldns_dname_label(sub_clone, j);
+			tmp_par = ldns_dname_label(parent_clone, i);
+			if (!tmp_sub || !tmp_par) {
+				/* deep free does null check */
+				ldns_rdf_deep_free(tmp_sub);
+				ldns_rdf_deep_free(tmp_par);
+				result = false;
+				break;
+			}
 
-		if (ldns_rdf_compare(tmp_sub, tmp_par) != 0) {
-			/* they are not equal */
+			if (ldns_rdf_compare(tmp_sub, tmp_par) != 0) {
+				/* they are not equal */
+				ldns_rdf_deep_free(tmp_sub);
+				ldns_rdf_deep_free(tmp_par);
+				result = false;
+				break;
+			}
 			ldns_rdf_deep_free(tmp_sub);
 			ldns_rdf_deep_free(tmp_par);
-			return false;
+			j--;
 		}
-		ldns_rdf_deep_free(tmp_sub);
-		ldns_rdf_deep_free(tmp_par);
-		j--;
 	}
-	return true; 
+	ldns_rdf_deep_free(sub_clone);
+	ldns_rdf_deep_free(parent_clone);
+	return result;
 }
 
 int
@@ -331,10 +344,9 @@ ldns_dname_compare(const ldns_rdf *dname1, const ldns_rdf *dname2)
 	assert(ldns_rdf_get_type(dname1) == LDNS_RDF_TYPE_DNAME);
 	assert(ldns_rdf_get_type(dname2) == LDNS_RDF_TYPE_DNAME);
 
-	
 	lc1 = ldns_dname_label_count(dname1);
 	lc2 = ldns_dname_label_count(dname2);
-	
+
 	if (lc1 == 0 && lc2 == 0) {
 		return 0;
 	}
@@ -355,7 +367,7 @@ ldns_dname_compare(const ldns_rdf *dname1, const ldns_rdf *dname2)
 			lp1 += *lp1 + 1;
 			lc1f--;
 		}
-		
+
 		/* and find the other one */
 		lc2f = lc2;
 		lp2 = ldns_rdf_data(dname2);
@@ -363,7 +375,7 @@ ldns_dname_compare(const ldns_rdf *dname1, const ldns_rdf *dname2)
 			lp2 += *lp2 + 1;
 			lc2f--;
 		}
-		
+
 		/* now check the label character for character. */
 		for (i = 1; i < (size_t)(*lp1 + 1); i++) {
 			if (i > *lp2) {
@@ -399,8 +411,7 @@ ldns_dname_compare(const ldns_rdf *dname1, const ldns_rdf *dname2)
 		lc1--;
 		lc2--;
 	}
-		
-	
+
 	done:
 	return result;
 }
@@ -426,13 +437,13 @@ ldns_dname_match_wildcard(const ldns_rdf *dname, const ldns_rdf *wildcard)
 	return result;
 }
 
-/* nsec test: does prev <= middle < next 
+/* nsec test: does prev <= middle < next
  * -1 = yes
  * 0 = error/can't tell
  * 1 = no
  */
 int
-ldns_dname_interval(const ldns_rdf *prev, const ldns_rdf *middle, 
+ldns_dname_interval(const ldns_rdf *prev, const ldns_rdf *middle,
 		const ldns_rdf *next)
 {
 	int prev_check, next_check;
@@ -456,7 +467,7 @@ ldns_dname_interval(const ldns_rdf *prev, const ldns_rdf *middle,
 			next_check == -1) {
 		return -1;
 	} else {
-		return 1; 
+		return 1;
 	}
 }
 
@@ -466,8 +477,8 @@ ldns_dname_str_absolute(const char *dname_str)
 {
 	if(dname_str && strcmp(dname_str, ".") == 0)
 		return 1;
-	return (dname_str && 
-	        strlen(dname_str) > 1 && 
+	return (dname_str &&
+	        strlen(dname_str) > 1 &&
 	        dname_str[strlen(dname_str) - 1] == '.' &&
 	        dname_str[strlen(dname_str) - 2] != '\\');
 }
@@ -480,15 +491,15 @@ ldns_dname_label(const ldns_rdf *rdf, uint8_t labelpos)
 	uint16_t len;
 	ldns_rdf *tmpnew;
 	size_t s;
-	
+
 	if (ldns_rdf_get_type(rdf) != LDNS_RDF_TYPE_DNAME) {
 		return NULL;
 	}
 
-	labelcnt = 0; 
+	labelcnt = 0;
 	src_pos = 0;
 	s = ldns_rdf_size(rdf);
-	
+
 	len = ldns_rdf_data(rdf)[src_pos]; /* label start */
 	while ((len > 0) && src_pos < s) {
 		if (labelcnt == labelpos) {
