@@ -1,4 +1,4 @@
-/* 
+/*
  * dnssec.c
  *
  * contains the cryptographic function needed for DNSSEC in ldns
@@ -32,11 +32,11 @@ ldns_dnssec_get_rrsig_for_name_and_type(const ldns_rdf *name,
 {
 	size_t i;
 	ldns_rr *candidate;
-	
+
 	if (!name || !rrs) {
 		return NULL;
 	}
-	
+
 	for (i = 0; i < ldns_rr_list_rr_count(rrs); i++) {
 		candidate = ldns_rr_list_rr(rrs, i);
 		if (ldns_rr_get_type(candidate) == LDNS_RR_TYPE_RRSIG) {
@@ -49,7 +49,7 @@ ldns_dnssec_get_rrsig_for_name_and_type(const ldns_rdf *name,
 			}
 		}
 	}
-	
+
 	return NULL;
 }
 
@@ -59,11 +59,11 @@ ldns_dnssec_get_dnskey_for_rrsig(const ldns_rr *rrsig,
 {
 	size_t i;
 	ldns_rr *candidate;
-	
+
 	if (!rrsig || !rrs) {
 		return NULL;
 	}
-	
+
 	for (i = 0; i < ldns_rr_list_rr_count(rrs); i++) {
 		candidate = ldns_rr_list_rr(rrs, i);
 		if (ldns_rr_get_type(candidate) == LDNS_RR_TYPE_DNSKEY) {
@@ -76,7 +76,7 @@ ldns_dnssec_get_dnskey_for_rrsig(const ldns_rr *rrsig,
 			}
 		}
 	}
-	
+
 	return NULL;
 }
 
@@ -107,18 +107,18 @@ ldns_dnssec_nsec3_closest_encloser(ldns_rdf *qname,
 	ldns_rdf *sname, *hashed_sname, *tmp;
 	ldns_rr *ce;
 	bool flag;
-	
+
 	bool exact_match_found;
 	bool in_range_found;
-	
+
 	ldns_status status;
 	ldns_rdf *zone_name;
-	
+
 	size_t nsec_i;
 	ldns_rr *nsec;
 	ldns_rdf *result = NULL;
 	qtype = qtype;
-	
+
 	if (!qname || !nsec3s || ldns_rr_list_rr_count(nsec3s) < 1) {
 		return NULL;
 	}
@@ -133,14 +133,14 @@ ldns_dnssec_nsec3_closest_encloser(ldns_rdf *qname,
 
 	ce = NULL;
 	flag = false;
-	
+
 	zone_name = ldns_dname_left_chop(ldns_rr_owner(nsec));
 
 	/* algorithm from nsec3-07 8.3 */
 	while (ldns_dname_label_count(sname) > 0) {
 		exact_match_found = false;
 		in_range_found = false;
-		
+
 		hashed_sname = ldns_nsec3_hash_name(sname,
 									 algorithm,
 									 iterations,
@@ -151,21 +151,24 @@ ldns_dnssec_nsec3_closest_encloser(ldns_rdf *qname,
 
 		for (nsec_i = 0; nsec_i < ldns_rr_list_rr_count(nsec3s); nsec_i++) {
 			nsec = ldns_rr_list_rr(nsec3s, nsec_i);
-			
+
 			/* check values of iterations etc! */
-			
+
 			/* exact match? */
 			if (ldns_dname_compare(ldns_rr_owner(nsec), hashed_sname) == 0) {
 			 	exact_match_found = true;
 			} else if (ldns_nsec_covers_name(nsec, hashed_sname)) {
 				in_range_found = true;
 			}
-			
+
 		}
 		if (!exact_match_found && in_range_found) {
 			flag = true;
 		} else if (exact_match_found && flag) {
 			result = ldns_rdf_clone(sname);
+			/* RFC 5155: 8.3. 2.** "The proof is complete" */
+			ldns_rdf_deep_free(hashed_sname);
+			goto done;
 		} else if (exact_match_found && !flag) {
 			/* error! */
 			ldns_rdf_deep_free(hashed_sname);
@@ -173,7 +176,7 @@ ldns_dnssec_nsec3_closest_encloser(ldns_rdf *qname,
 		} else {
 			flag = false;
 		}
-		
+
 		ldns_rdf_deep_free(hashed_sname);
 		tmp = sname;
 		sname = ldns_dname_left_chop(sname);
@@ -241,7 +244,7 @@ ldns_dnssec_pkt_get_rrsigs_for_type(const ldns_pkt *pkt, ldns_rr_type type)
 	ldns_rr_list *sigs;
 	ldns_rr_list *sigs_covered;
 	ldns_rdf *rdf_t;
-	
+
 	sigs = ldns_pkt_rr_list_by_type(pkt,
 	                                LDNS_RR_TYPE_RRSIG,
 	                                LDNS_SECTION_ANY_NOQUESTION
@@ -252,7 +255,7 @@ ldns_dnssec_pkt_get_rrsigs_for_type(const ldns_pkt *pkt, ldns_rr_type type)
 					 2,
 					 &t_netorder);
 	sigs_covered = ldns_rr_list_subtype_by_rdf(sigs, rdf_t, 0);
-	
+
 	ldns_rdf_free(rdf_t);
 	ldns_rr_list_deep_free(sigs);
 
@@ -342,27 +345,33 @@ ldns_key_buf2dsa_raw(unsigned char* key, size_t len)
 	T = (uint8_t)key[0];
 	length = (64 + T * 8);
 	offset = 1;
-	
+
 	if (T > 8) {
 		return NULL;
 	}
 	if(len < (size_t)1 + SHA_DIGEST_LENGTH + 3*length)
 		return NULL;
-	
+
 	Q = BN_bin2bn(key+offset, SHA_DIGEST_LENGTH, NULL);
 	offset += SHA_DIGEST_LENGTH;
-	
+
 	P = BN_bin2bn(key+offset, (int)length, NULL);
 	offset += length;
-	
+
 	G = BN_bin2bn(key+offset, (int)length, NULL);
 	offset += length;
-	
+
 	Y = BN_bin2bn(key+offset, (int)length, NULL);
 	offset += length;
-	
+
 	/* create the key and set its properties */
-	dsa = DSA_new();
+	if(!Q || !P || !G || !Y || !(dsa = DSA_new())) {
+		BN_free(Q);
+		BN_free(P);
+		BN_free(G);
+		BN_free(Y);
+		return NULL;
+	}
 	dsa->p = P;
 	dsa->q = Q;
 	dsa->g = G;
@@ -407,18 +416,28 @@ ldns_key_buf2rsa_raw(unsigned char* key, size_t len)
 	/* key length at least one */
 	if(len < (size_t)offset + exp + 1)
 		return NULL;
-	
+
 	/* Exponent */
 	exponent = BN_new();
+	if(!exponent) return NULL;
 	(void) BN_bin2bn(key+offset, (int)exp, exponent);
 	offset += exp;
 
 	/* Modulus */
 	modulus = BN_new();
+	if(!modulus) {
+		BN_free(exponent);
+		return NULL;
+	}
 	/* length of the buffer must match the key length! */
 	(void) BN_bin2bn(key+offset, (int)(len - offset), modulus);
 
 	rsa = RSA_new();
+	if(!rsa) {
+		BN_free(exponent);
+		BN_free(modulus);
+		return NULL;
+	}
 	rsa->n = modulus;
 	rsa->e = exponent;
 
@@ -431,7 +450,7 @@ ldns_digest_evp(unsigned char* data, unsigned int len, unsigned char* dest,
 {
 	EVP_MD_CTX* ctx;
 	ctx = EVP_MD_CTX_create();
-	if(!ctx) 
+	if(!ctx)
 		return false;
 	if(!EVP_DigestInit_ex(ctx, md, NULL) ||
 		!EVP_DigestUpdate(ctx, data, len) ||
@@ -547,7 +566,7 @@ ldns_key_rr2ds(const ldns_rr *key, ldns_hash h)
 
 	/* all the rdata's */
 	if (ldns_rr_rdata2buffer_wire(data_buf,
-							(ldns_rr*)key) != LDNS_STATUS_OK) { 
+							(ldns_rr*)key) != LDNS_STATUS_OK) {
 		LDNS_FREE(digest);
 		ldns_buffer_free(data_buf);
 		ldns_rr_free(ds);
@@ -619,12 +638,15 @@ ldns_dnssec_create_nsec_bitmap(ldns_rr_type rr_type_list[],
 		return NULL;
 	}
 
-	/* the types in the list should be orders, lowest first,
-	   so the last one contains the highest type */
-	i_type = rr_type_list[size-1];
+	i_type = 0;
+	for (i = 0; i < size; i++) {
+		if (i_type < rr_type_list[i])
+			i_type = rr_type_list[i];
+	}
 	if (i_type < nsec_type) {
 		i_type = nsec_type;
 	}
+
 	bm_len = i_type / 8 + 2;
 	bitmap = LDNS_XMALLOC(uint8_t, bm_len);
 	for (i = 0; i < bm_len; i++) {
@@ -674,7 +696,7 @@ ldns_dnssec_create_nsec_bitmap(ldns_rr_type rr_type_list[],
 		memcpy(data + cur_data_size + 2, cur_data, cur_window_max+1);
 		cur_data_size += cur_window_max + 3;
 	}
-	
+
 	bitmap_rdf = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_NSEC,
 								cur_data_size,
 								data);
@@ -719,7 +741,7 @@ ldns_dnssec_create_nsec(ldns_dnssec_name *from,
                         ldns_rr_type nsec_type)
 {
 	ldns_rr *nsec_rr;
-	ldns_rr_type types[1024];
+	ldns_rr_type types[65535];
 	size_t type_count = 0;
 	ldns_dnssec_rrsets *cur_rrsets;
 
@@ -766,7 +788,7 @@ ldns_dnssec_create_nsec3(ldns_dnssec_name *from,
 					uint8_t *salt)
 {
 	ldns_rr *nsec_rr;
-	ldns_rr_type types[1024];
+	ldns_rr_type types[65535];
 	size_t type_count = 0;
 	ldns_dnssec_rrsets *cur_rrsets;
 	ldns_status status;
@@ -802,7 +824,7 @@ ldns_dnssec_create_nsec3(ldns_dnssec_name *from,
 		type_count++;
 		cur_rrsets = cur_rrsets->next;
 	}
-	/* always add rrsig type if this is not an unsigned 
+	/* always add rrsig type if this is not an unsigned
 	 * delegation
 	 */
 	if (type_count > 0 &&
@@ -810,7 +832,7 @@ ldns_dnssec_create_nsec3(ldns_dnssec_name *from,
 		types[type_count] = LDNS_RR_TYPE_RRSIG;
 		type_count++;
 	}
-	
+
 	/* leave next rdata empty if they weren't precomputed yet */
 	if (to && to->hashed_name) {
 		(void) ldns_rr_set_rdf(nsec_rr,
@@ -832,7 +854,7 @@ ldns_rr *
 ldns_create_nsec(ldns_rdf *cur_owner, ldns_rdf *next_owner, ldns_rr_list *rrs)
 {
 	/* we do not do any check here - garbage in, garbage out */
-	
+
 	/* the the start and end names - get the type from the
 	 * before rrlist */
 
@@ -841,109 +863,38 @@ ldns_create_nsec(ldns_rdf *cur_owner, ldns_rdf *next_owner, ldns_rr_list *rrs)
 	/* todo: make something more efficient :) */
 	uint16_t i;
 	ldns_rr *i_rr;
-
-	uint8_t *bitmap = LDNS_XMALLOC(uint8_t, 2);
-	uint16_t bm_len = 0;
 	uint16_t i_type;
 
 	ldns_rr *nsec = NULL;
-
-	uint8_t *data = NULL;
-	uint8_t cur_data[32];
-	uint8_t cur_window = 0;
-	uint8_t cur_window_max = 0;
-	uint16_t cur_data_size = 0;
+    ldns_rr_type i_type_list[65535];
+	int type_count = 0;
 
 	nsec = ldns_rr_new();
 	ldns_rr_set_type(nsec, LDNS_RR_TYPE_NSEC);
 	ldns_rr_set_owner(nsec, ldns_rdf_clone(cur_owner));
 	ldns_rr_push_rdf(nsec, ldns_rdf_clone(next_owner));
 
-	bitmap[0] = 0;
 	for (i = 0; i < ldns_rr_list_rr_count(rrs); i++) {
 		i_rr = ldns_rr_list_rr(rrs, i);
-
 		if (ldns_rdf_compare(cur_owner,
 						 ldns_rr_owner(i_rr)) == 0) {
-			/* add type to bitmap */
 			i_type = ldns_rr_get_type(i_rr);
-			if ((i_type / 8) + 1 > bm_len) {
-				bitmap = LDNS_XREALLOC(bitmap, uint8_t, (i_type / 8) + 2);
-				/* set to 0 */
-				for (; bm_len <= i_type / 8; bm_len++) {
-					bitmap[bm_len] = 0;
-				}
+			if (type_count == 0 || i_type_list[type_count-1] != i_type) {
+				i_type_list[type_count] = i_type;
+				type_count++;
 			}
-			ldns_set_bit(bitmap + (int) i_type / 8,
-					   (int) (7 - (i_type % 8)),
-					   true);
 		}
 	}
-	/* add NSEC and RRSIG anyway */
-	i_type = LDNS_RR_TYPE_RRSIG;
-	if (i_type / 8 > bm_len) {
-		bitmap = LDNS_XREALLOC(bitmap, uint8_t, (i_type / 8) + 2);
-		/* set to 0 */
-		for (; bm_len <= i_type / 8; bm_len++) {
-			bitmap[bm_len] = 0;
-		}
-	}
-	ldns_set_bit(bitmap + (int) i_type / 8, (int) (7 - (i_type % 8)), true);
-	i_type = LDNS_RR_TYPE_NSEC;
 
-	if (i_type / 8 > bm_len) {
-		bitmap = LDNS_XREALLOC(bitmap, uint8_t, (i_type / 8) + 2);
-		/* set to 0 */
-		for (; bm_len <= i_type / 8; bm_len++) {
-			bitmap[bm_len] = 0;
-		}
-	}
-	ldns_set_bit(bitmap + (int) i_type / 8, (int) (7 - (i_type % 8)), true);
-
-	memset(cur_data, 0, 32);
-	for (i = 0; i < bm_len; i++) {
-		if (i / 32 > cur_window) {
-			/* check, copy, new */
-			if (cur_window_max > 0) {
-				/* this window has stuff, add it */
-				data = LDNS_XREALLOC(data,
-								 uint8_t,
-								 cur_data_size + cur_window_max + 3);
-				data[cur_data_size] = cur_window;
-				data[cur_data_size + 1] = cur_window_max + 1;
-				memcpy(data + cur_data_size + 2,
-					  cur_data,
-					  cur_window_max+1);
-				cur_data_size += cur_window_max + 3;
-			}
-			cur_window++;
-			cur_window_max = 0;
-			memset(cur_data, 0, 32);
-		} else {
-			cur_data[i%32] = bitmap[i];
-			if (bitmap[i] > 0) {
-				cur_window_max = i%32;
-			}
-		}
-	}
-	if (cur_window_max > 0) {
-		/* this window has stuff, add it */
-		data = LDNS_XREALLOC(data,
-						 uint8_t,
-						 cur_data_size + cur_window_max + 3);
-		data[cur_data_size] = cur_window;
-		data[cur_data_size + 1] = cur_window_max + 1;
-		memcpy(data + cur_data_size + 2, cur_data, cur_window_max+1);
-		cur_data_size += cur_window_max + 3;
-	}
+	i_type_list[type_count] = LDNS_RR_TYPE_RRSIG;
+	type_count++;
+	i_type_list[type_count] = LDNS_RR_TYPE_NSEC;
+	type_count++;
 
 	ldns_rr_push_rdf(nsec,
-				  ldns_rdf_new_frm_data(LDNS_RDF_TYPE_NSEC,
-								    cur_data_size,
-								    data));
+				  ldns_dnssec_create_nsec_bitmap(i_type_list,
+						type_count, LDNS_RR_TYPE_NSEC));
 
-	LDNS_FREE(bitmap);
-	LDNS_FREE(data);
 	return nsec;
 }
 
@@ -954,8 +905,8 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 				 uint8_t salt_length,
 				 uint8_t *salt)
 {
-	char *orig_owner_str;
 	size_t hashed_owner_str_len;
+	ldns_rdf *cann;
 	ldns_rdf *hashed_owner;
 	unsigned char *hashed_owner_str;
 	char *hashed_owner_b32;
@@ -965,17 +916,23 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 	 * sha1 at the moment */
 	unsigned char hash[LDNS_SHA1_DIGEST_LENGTH];
 	ldns_status status;
-	
+
 	/* prepare the owner name according to the draft section bla */
-	orig_owner_str = ldns_rdf2str(name);
-	
+	cann = ldns_rdf_clone(name);
+	if(!cann) {
+		fprintf(stderr, "Memory error\n");
+		return NULL;
+	}
+	ldns_dname2canonical(cann);
+
 	/* TODO: mnemonic list for hash algs SHA-1, default to 1 now (sha1) */
 	algorithm = algorithm;
-	
-	hashed_owner_str_len = salt_length + ldns_rdf_size(name);
+
+	hashed_owner_str_len = salt_length + ldns_rdf_size(cann);
 	hashed_owner_str = LDNS_XMALLOC(unsigned char, hashed_owner_str_len);
-	memcpy(hashed_owner_str, ldns_rdf_data(name), ldns_rdf_size(name));
-	memcpy(hashed_owner_str + ldns_rdf_size(name), salt, salt_length);
+	memcpy(hashed_owner_str, ldns_rdf_data(cann), ldns_rdf_size(cann));
+	memcpy(hashed_owner_str + ldns_rdf_size(cann), salt, salt_length);
+	ldns_rdf_deep_free(cann);
 
 	for (cur_it = iterations + 1; cur_it > 0; cur_it--) {
 		(void) ldns_sha1((unsigned char *) hashed_owner_str,
@@ -986,27 +943,24 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 		hashed_owner_str = LDNS_XMALLOC(unsigned char, hashed_owner_str_len);
 		if (!hashed_owner_str) {
 			fprintf(stderr, "Memory error\n");
-			abort();
+			return NULL;
 		}
 		memcpy(hashed_owner_str, hash, LDNS_SHA1_DIGEST_LENGTH);
 		memcpy(hashed_owner_str + LDNS_SHA1_DIGEST_LENGTH, salt, salt_length);
 		hashed_owner_str_len = LDNS_SHA1_DIGEST_LENGTH + salt_length;
 	}
 
-	LDNS_FREE(orig_owner_str);
 	LDNS_FREE(hashed_owner_str);
 	hashed_owner_str = hash;
 	hashed_owner_str_len = LDNS_SHA1_DIGEST_LENGTH;
 
 	hashed_owner_b32 = LDNS_XMALLOC(char,
-							  ldns_b32_ntop_calculate_size(
-							       hashed_owner_str_len) + 1);
-	hashed_owner_b32_len =
-		(size_t) ldns_b32_ntop_extended_hex((uint8_t *) hashed_owner_str,
-									 hashed_owner_str_len,
-									 hashed_owner_b32,
-									 ldns_b32_ntop_calculate_size(
-										 hashed_owner_str_len));
+                  ldns_b32_ntop_calculate_size(hashed_owner_str_len) + 1);
+        hashed_owner_b32_len = (size_t) ldns_b32_ntop_extended_hex(
+                (uint8_t *) hashed_owner_str,
+                hashed_owner_str_len,
+                hashed_owner_b32,
+                ldns_b32_ntop_calculate_size(hashed_owner_str_len));
 	if (hashed_owner_b32_len < 1) {
 		fprintf(stderr, "Error in base32 extended hex encoding ");
 		fprintf(stderr, "of hashed owner name (name: ");
@@ -1032,7 +986,7 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 
 void
 ldns_nsec3_add_param_rdfs(ldns_rr *rr,
-					 uint8_t algorithm, 
+					 uint8_t algorithm,
 					 uint8_t flags,
 					 uint16_t iterations,
 					 uint8_t salt_length,
@@ -1053,13 +1007,13 @@ ldns_nsec3_add_param_rdfs(ldns_rr *rr,
 	                                            1, (void*)&flags),
 	                      1);
 	if (old) ldns_rdf_deep_free(old);
-	
+
 	old = ldns_rr_set_rdf(rr,
                           ldns_native2rdf_int16(LDNS_RDF_TYPE_INT16,
                                                 iterations),
 	                      2);
 	if (old) ldns_rdf_deep_free(old);
-	
+
 	salt_data = LDNS_XMALLOC(uint8_t, salt_length + 1);
 	salt_data[0] = salt_length;
 	memcpy(salt_data + 1, salt, salt_length);
@@ -1105,34 +1059,27 @@ ldns_create_nsec3(ldns_rdf *cur_owner,
 {
 	size_t i;
 	ldns_rr *i_rr;
-
-	uint8_t *bitmap = LDNS_XMALLOC(uint8_t, 1);
-	uint16_t bm_len = 0;
 	uint16_t i_type;
 
 	ldns_rr *nsec = NULL;
 	ldns_rdf *hashed_owner = NULL;
-	
-	uint8_t *data = NULL;
-	uint8_t cur_data[32];
-	uint8_t cur_window = 0;
-	uint8_t cur_window_max = 0;
-	uint16_t cur_data_size = 0;
 
 	ldns_status status;
 
-	
+    ldns_rr_type i_type_list[1024];
+	int type_count = 0;
+
 	hashed_owner = ldns_nsec3_hash_name(cur_owner,
 								 algorithm,
 								 iterations,
 								 salt_length,
 								 salt);
 	status = ldns_dname_cat(hashed_owner, cur_zone);
-	
+
 	nsec = ldns_rr_new_frm_type(LDNS_RR_TYPE_NSEC3);
 	ldns_rr_set_type(nsec, LDNS_RR_TYPE_NSEC3);
 	ldns_rr_set_owner(nsec, hashed_owner);
-	
+
 	ldns_nsec3_add_param_rdfs(nsec,
 						 algorithm,
 						 flags,
@@ -1141,98 +1088,35 @@ ldns_create_nsec3(ldns_rdf *cur_owner,
 						 salt);
 	(void) ldns_rr_set_rdf(nsec, NULL, 4);
 
-	bitmap[0] = 0;
+
 	for (i = 0; i < ldns_rr_list_rr_count(rrs); i++) {
 		i_rr = ldns_rr_list_rr(rrs, i);
-
 		if (ldns_rdf_compare(cur_owner,
-		                     ldns_rr_owner(i_rr)) == 0) {
-			/* add type to bitmap */
+						 ldns_rr_owner(i_rr)) == 0) {
 			i_type = ldns_rr_get_type(i_rr);
-			if ((i_type / 8) + 1 > bm_len) {
-				bitmap = LDNS_XREALLOC(bitmap, uint8_t, (i_type / 8) + 1);
-				/* set to 0 */
-				for (; bm_len <= i_type / 8; bm_len++) {
-					bitmap[bm_len] = 0;
-				}
+			if (type_count == 0 || i_type_list[type_count-1] != i_type) {
+				i_type_list[type_count] = i_type;
+				type_count++;
 			}
-			ldns_set_bit(bitmap + (int) i_type / 8,
-					   (int) (7 - (i_type % 8)),
-					   true);
 		}
 	}
-	/* add NSEC and RRSIG anyway, but only if this is not an ENT or
+
+	/* add RRSIG anyway, but only if this is not an ENT or
 	 * an unsigned delegation */
 	if (!emptynonterminal && !rr_list_delegation_only(cur_zone, rrs)) {
-		i_type = LDNS_RR_TYPE_RRSIG;
-		if (i_type / 8 > bm_len) {
-			bitmap = LDNS_XREALLOC(bitmap, uint8_t, (i_type / 8) + 1);
-			/* set to 0 */
-			for (; bm_len <= i_type / 8; bm_len++) {
-				bitmap[bm_len] = 0;
-			}
-		}
-		ldns_set_bit(bitmap + (int) i_type / 8,
-				   (int) (7 - (i_type % 8)),
-				   true);
+		i_type_list[type_count] = LDNS_RR_TYPE_RRSIG;
+		type_count++;
 	}
 
 	/* and SOA if owner == zone */
 	if (ldns_dname_compare(cur_zone, cur_owner) == 0) {
-		i_type = LDNS_RR_TYPE_SOA;
-		if (i_type / 8 > bm_len) {
-			bitmap = LDNS_XREALLOC(bitmap, uint8_t, (i_type / 8) + 1);
-			/* set to 0 */
-			for (; bm_len <= i_type / 8; bm_len++) {
-				bitmap[bm_len] = 0;
-			}
-		}
-		ldns_set_bit(bitmap + (int) i_type / 8,
-				   (int) (7 - (i_type % 8)),
-				   true);
+		i_type_list[type_count] = LDNS_RR_TYPE_SOA;
+		type_count++;
 	}
 
-	memset(cur_data, 0, 32);
-	for (i = 0; i < bm_len; i++) {
-		if (i / 32 > cur_window) {
-			/* check, copy, new */
-			if (cur_window_max > 0) {
-				/* this window has stuff, add it */
-				data = LDNS_XREALLOC(data,
-								 uint8_t,
-								 cur_data_size + cur_window_max + 3);
-				data[cur_data_size] = cur_window;
-				data[cur_data_size + 1] = cur_window_max + 1;
-				memcpy(data + cur_data_size + 2,
-					  cur_data,
-					  cur_window_max+1);
-				cur_data_size += cur_window_max + 3;
-			}
-			cur_window++;
-			cur_window_max = 0;
-			memset(cur_data, 0, 32);
-		} else {
-			cur_data[i%32] = bitmap[i];
-			if (bitmap[i] > 0) {
-				cur_window_max = i%32;
-			}
-		}
-	}
-	data = LDNS_XREALLOC(data,
-					 uint8_t,
-					 cur_data_size + cur_window_max + 3);
-	data[cur_data_size] = cur_window;
-	data[cur_data_size + 1] = cur_window_max + 1;
-	memcpy(data + cur_data_size + 2, cur_data, cur_window_max+1);
-	cur_data_size += cur_window_max + 3;
-
 	ldns_rr_push_rdf(nsec,
-	                 ldns_rdf_new_frm_data(LDNS_RDF_TYPE_NSEC,
-	                                       cur_data_size,
-	                                       data));
-
-	LDNS_FREE(bitmap);
-	LDNS_FREE(data);
+				  ldns_dnssec_create_nsec_bitmap(i_type_list,
+						type_count, LDNS_RR_TYPE_NSEC3));
 
 	return nsec;
 }
@@ -1340,20 +1224,20 @@ ldns_nsec3_hash_name_frm_nsec3(const ldns_rr *nsec, ldns_rdf *name)
 	uint16_t iterations;
 	uint8_t salt_length;
 	uint8_t *salt = 0;
-	
+
 	ldns_rdf *hashed_owner;
 
 	algorithm = ldns_nsec3_algorithm(nsec);
 	salt_length = ldns_nsec3_salt_length(nsec);
 	salt = ldns_nsec3_salt_data(nsec);
 	iterations = ldns_nsec3_iterations(nsec);
-	
+
 	hashed_owner = ldns_nsec3_hash_name(name,
 								 algorithm,
 								 iterations,
 								 salt_length,
 								 salt);
-	
+
 	LDNS_FREE(salt);
 	return hashed_owner;
 }
@@ -1367,12 +1251,12 @@ ldns_nsec_bitmap_covers_type(const ldns_rdf *nsec_bitmap, ldns_rr_type type)
 	uint16_t pos = 0;
 	uint16_t bit_pos;
 	uint8_t *data = ldns_rdf_data(nsec_bitmap);
-	
+
 	while(pos < ldns_rdf_size(nsec_bitmap)) {
 		window_block_nr = data[pos];
 		bitmap_length = data[pos + 1];
 		pos += 2;
-		
+
 		for (bit_pos = 0; bit_pos < (bitmap_length) * 8; bit_pos++) {
 			if (ldns_get_bit(&data[pos], bit_pos)) {
 				cur_type = 256 * (uint16_t) window_block_nr + bit_pos;
@@ -1381,7 +1265,7 @@ ldns_nsec_bitmap_covers_type(const ldns_rdf *nsec_bitmap, ldns_rr_type type)
 				}
 			}
 		}
-		
+
 		pos += (uint16_t) bitmap_length;
 	}
 	return false;
@@ -1397,7 +1281,7 @@ ldns_nsec_covers_name(const ldns_rr *nsec, const ldns_rdf *name)
 	ldns_status status;
 	ldns_rdf *chopped_dname;
 	bool result;
-	
+
 	if (ldns_rr_get_type(nsec) == LDNS_RR_TYPE_NSEC) {
 		nsec_next = ldns_rdf_clone(ldns_rr_rdf(nsec, 0));
 	} else if (ldns_rr_get_type(nsec) == LDNS_RR_TYPE_NSEC3) {
@@ -1415,7 +1299,7 @@ ldns_nsec_covers_name(const ldns_rr *nsec, const ldns_rdf *name)
 		ldns_rdf_deep_free(nsec_next);
 		return false;
 	}
-	
+
 	/* in the case of the last nsec */
 	if(ldns_dname_compare(nsec_owner, nsec_next) > 0) {
 		result = (ldns_dname_compare(nsec_owner, name) <= 0 ||
@@ -1424,7 +1308,7 @@ ldns_nsec_covers_name(const ldns_rr *nsec, const ldns_rdf *name)
 		result = (ldns_dname_compare(nsec_owner, name) <= 0 &&
 		          ldns_dname_compare(name, nsec_next) < 0);
 	}
-	
+
 	ldns_rdf_deep_free(nsec_next);
 	return result;
 }
@@ -1432,7 +1316,7 @@ ldns_nsec_covers_name(const ldns_rr *nsec, const ldns_rdf *name)
 #ifdef HAVE_SSL
 /* sig may be null - if so look in the packet */
 ldns_status
-ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o, 
+ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o,
 			 ldns_rr_list *k, ldns_rr_list *s, ldns_rr_list *good_keys)
 {
 	ldns_rr_list *rrset;
@@ -1450,13 +1334,13 @@ ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o,
 		/* we don't have RRSIG(RRSIG) (yet? ;-) ) */
 		return LDNS_STATUS_ERR;
 	}
-	
+
 	if (s) {
 		/* if s is not NULL, the sigs are given to use */
 		sigs = s;
 	} else {
 		/* otherwise get them from the packet */
-		sigs = ldns_pkt_rr_list_by_name_and_type(p, o, LDNS_RR_TYPE_RRSIG, 
+		sigs = ldns_pkt_rr_list_by_name_and_type(p, o, LDNS_RR_TYPE_RRSIG,
 									  LDNS_SECTION_ANY_NOQUESTION);
 		if (!sigs) {
 			/* no sigs */
@@ -1474,7 +1358,7 @@ ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o,
 					 2,
 					 &t_netorder);
 	sigs_covered = ldns_rr_list_subtype_by_rdf(sigs, rdf_t, 0);
-	
+
 	rrset = ldns_pkt_rr_list_by_name_and_type(p,
 									  o,
 									  t,
@@ -1503,7 +1387,7 @@ ldns_dnssec_chain_nsec3_list(ldns_rr_list *nsec3_rrs)
 
 	for (i = 0; i < ldns_rr_list_rr_count(nsec3_rrs); i++) {
 		if (i == ldns_rr_list_rr_count(nsec3_rrs) - 1) {
-			next_nsec_owner_label = 
+			next_nsec_owner_label =
 				ldns_dname_label(ldns_rr_owner(ldns_rr_list_rr(nsec3_rrs,
 													  0)), 0);
 			next_nsec_owner_str = ldns_rdf2str(next_nsec_owner_label);
@@ -1522,7 +1406,7 @@ ldns_dnssec_chain_nsec3_list(ldns_rr_list *nsec3_rrs)
 			ldns_rdf_deep_free(next_nsec_owner_label);
 			LDNS_FREE(next_nsec_owner_str);
 		} else {
-			next_nsec_owner_label = 
+			next_nsec_owner_label =
 				ldns_dname_label(ldns_rr_owner(ldns_rr_list_rr(nsec3_rrs,
 													  i + 1)),
 							  0);
@@ -1555,7 +1439,7 @@ qsort_rr_compare_nsec3(const void *a, const void *b)
 	}
 	if (rr1 == NULL) {
 		return -1;
-	} 
+	}
 	if (rr2 == NULL) {
 		return 1;
 	}
@@ -1634,7 +1518,7 @@ ldns_convert_dsa_rrsig_asn12rdf(const ldns_buffer *sig,
 	}
 	memset(&dsasig_data[21], 0, byte_offset);
 	BN_bn2bin(dsasig->s, &dsasig_data[21 + byte_offset]);
-	
+
 	sigdata_rdf = ldns_rdf_new(LDNS_RDF_TYPE_B64, 41, dsasig_data);
 	DSA_SIG_free(dsasig);
 
@@ -1651,24 +1535,31 @@ ldns_convert_dsa_rrsig_rdf2asn1(ldns_buffer *target_buffer,
 	DSA_SIG *dsasig;
 	unsigned char *raw_sig = NULL;
 	int raw_sig_len;
-	
+
 	/* extract the R and S field from the sig buffer */
 	t = ldns_rdf_data(sig_rdf)[0];
 	R = BN_new();
+	if(!R) return LDNS_STATUS_MEM_ERR;
 	(void) BN_bin2bn((unsigned char *) ldns_rdf_data(sig_rdf) + 1,
 	                 SHA_DIGEST_LENGTH, R);
 	S = BN_new();
+	if(!S) {
+		BN_free(R);
+		return LDNS_STATUS_MEM_ERR;
+	}
 	(void) BN_bin2bn((unsigned char *) ldns_rdf_data(sig_rdf) + 21,
 	                 SHA_DIGEST_LENGTH, S);
 
 	dsasig = DSA_SIG_new();
 	if (!dsasig) {
+		BN_free(R);
+		BN_free(S);
 		return LDNS_STATUS_MEM_ERR;
 	}
 
 	dsasig->r = R;
 	dsasig->s = S;
-	
+
 	raw_sig_len = i2d_DSA_SIG(dsasig, &raw_sig);
 	if (raw_sig_len < 0) {
 		DSA_SIG_free(dsasig);
