@@ -320,16 +320,6 @@ get_match_name_str(match_id id) {
 	}
 }
 
-static bool
-is_match_name(char *name) {
-	match_table *mt = get_match_by_name(name);
-	if (mt) {
-		return true;
-	} else {
-		return false;
-	}
-}
-
 static void
 print_match_operation(FILE *output, match_operation *mc)
 {
@@ -453,25 +443,6 @@ fprintf(output, "_");
 	}			
 }
 
-static size_t 
-calculate_counters_total(match_counters *counters)
-{
-	size_t result = 0;
-	if (counters) {
-		if (counters->left) {
-			result += calculate_counters_total(counters->left);
-		}
-		if (counters->match) {
-			result += counters->match->count;
-		}
-		if (counters->right) {
-			result += calculate_counters_total(counters->left);
-		}
-	}
-	
-	return result;
-}
-
 static void
 print_counters(FILE *output, match_counters *counters, bool show_percentages, size_t total, int count_minimum)
 {
@@ -1353,45 +1324,6 @@ match_packet_to_operation(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr,
 	}
 }
 
-static int
-match_expression_compare_count(const void *a, const void *b)
-{
-	match_expression *mea, *meb;
-	
-	if (!a) {
-		return 1;
-	} else if (!b) {
-		return -1;
-	} else {
-		mea = (match_expression *) a;
-		meb = (match_expression *) b;
-		
-		if (mea->count < meb->count) {
-			return -1;
-		} else if (mea->count > meb->count) {
-			return 1;
-		} else {
-			return 0;
-		}
-	}
-}
-
-static int
-match_expression_compare_count_p(const void *a, const void *b)
-{
-	match_expression **pmea, **pmeb;
-	
-	if (!a) {
-		return 1;
-	} else if (!b) {
-		return -1;
-	} else {
-		pmea = (match_expression **) a;
-		pmeb = (match_expression **) b;
-		return match_expression_compare_count(*pmea, *pmeb);
-	}
-}
-
 static int
 match_operation_compare(const void *a, const void *b)
 {
@@ -1484,23 +1416,6 @@ match_expression_compare(const void *a, const void *b)
 	}
 }
 
-static int
-match_expression_compare_p(const void *a, const void *b)
-{
-	
-	match_expression **pmea, **pmeb;
-	
-	if (!a) {
-		return 1;
-	} else if (!b) {
-		return -1;
-	} else {
-		pmea = (match_expression **) a;
-		pmeb = (match_expression **) b;
-		return match_expression_compare(*pmea, *pmeb);
-	}
-}
-
 /**
  * If count is true, and the counter is found, its count is increased by 1
  */
@@ -1600,37 +1515,6 @@ match_dns_packet_to_expr(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr,
 	return result;
 }
 
-static bool
-match_expression_equals(match_expression *expr1, match_expression *expr2)
-{
-	if (!expr1 || !expr2) {
-		return false;
-	}
-
-	switch(expr1->op) {
-		case MATCH_EXPR_OR:
-		case MATCH_EXPR_AND:
-			if (!expr2->left || !expr2->right) {
-				return false;
-			}
-			return (match_expression_equals(expr1->left, expr2->left) &&
-			        match_expression_equals(expr1->right, expr2->right)
-			       );
-			break;
-		case MATCH_EXPR_LEAF:
-			if (!expr2->match) {
-				return false;
-			}
-			return expr1->match->id == expr2->match->id &&
-			       expr1->match->operator == expr2->match->operator &&
-			       strcmp(expr1->match->value, expr2->match->value) == 0;
-			break;
-		default:
-			return false;
-	}
-
-}
-
 static void
 free_match_operation(match_operation *operation)
 {
