@@ -29,6 +29,7 @@
 #endif
 #include <sys/time.h>
 #include <errno.h>
+#include <fcntl.h>
 
 ldns_status
 ldns_send(ldns_pkt **result_packet, ldns_resolver *r, const ldns_pkt *query_pkt)
@@ -218,6 +219,67 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 	return status;
 }
 
+/** best effort to set nonblocking */
+static void
+ldns_sock_nonblock(int sockfd)
+{
+#ifdef HAVE_FCNTL
+	int flag;
+	if((flag = fcntl(sockfd, F_GETFL)) != -1) {
+		flag |= O_NONBLOCK;
+		if(fcntl(sockfd, F_SETFL, flag) == -1) {
+			/* ignore error, continue blockingly */
+		}
+	}
+#elif defined(HAVE_IOCTLSOCKET)
+	unsigned long on = 1;
+	if(ioctlsocket(sockfd, FIONBIO, &on) != 0) {
+		/* ignore error, continue blockingly */
+	}
+#endif
+}
+
+/** best effort to set blocking */
+static void
+ldns_sock_block(int sockfd)
+{
+#ifdef HAVE_FCNTL
+	int flag;
+	if((flag = fcntl(sockfd, F_GETFL)) != -1) {
+		flag &= ~O_NONBLOCK;
+		if(fcntl(sockfd, F_SETFL, flag) == -1) {
+			/* ignore error, continue */
+		}
+	}
+#elif defined(HAVE_IOCTLSOCKET)
+	unsigned long off = 0;
+	if(ioctlsocket(sockfd, FIONBIO, &off) != 0) {
+		/* ignore error, continue */
+	}
+#endif
+}
+
+/** wait for a socket to become ready */
+static int
+ldns_sock_wait(int sockfd, struct timeval timeout, int write)
+{
+	fd_set fds;
+	int ret;
+	FD_ZERO(&fds);
+	FD_SET(FD_SET_T sockfd, &fds);
+	if(write)
+		ret = select(sockfd+1, NULL, &fds, NULL, &timeout);
+	else
+		ret = select(sockfd+1, &fds, NULL, NULL, &timeout);
+	if(ret == 0)
+		/* timeout expired */
+		return 0;
+	else if(ret == -1)
+		/* error */
+		return 0;
+	return 1;
+}
+
 ldns_status
 ldns_udp_send(uint8_t **result, ldns_buffer *qbin, const struct sockaddr_storage *to, 
 		socklen_t tolen, struct timeval timeout, size_t *answer_size)
@@ -232,6 +294,11 @@ ldns_udp_send(uint8_t **result, ldns_buffer *qbin, const struct sockaddr_storage
 	}
 
 	/* wait for an response*/
+	if(!ldns_sock_wait(sockfd, timeout, 0)) {
+		close(sockfd);
+		return LDNS_STATUS_NETWORK_ERR;
+	}
+	
 	answer = ldns_udp_read_wire(sockfd, answer_size, NULL, NULL);
 	close(sockfd);
 
@@ -263,7 +330,7 @@ ldns_udp_bgsend(ldns_buffer *qbin, const struct sockaddr_storage *to, socklen_t
 }
 
 int
-ldns_udp_connect(const struct sockaddr_storage *to, struct timeval timeout)
+ldns_udp_connect(const struct sockaddr_storage *to, struct timeval ATTR_UNUSED(timeout))
 {
 	int sockfd;
 	
@@ -272,14 +339,6 @@ ldns_udp_connect(const struct sockaddr_storage *to, struct timeval timeout)
 			== -1) {
                 return 0;
         }
-	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (void*)&timeout,
-				(socklen_t)sizeof(timeout))) {
-                /* might fail, in that case, use default for now */
-		/*
-		close(sockfd);
-		return 0;
-		*/
-        }
 	return sockfd;
 }
 
@@ -294,22 +353,76 @@ ldns_tcp_connect(const struct sockaddr_storage *to, socklen_t tolen,
 		return 0;
 	}
 
-	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (void*)&timeout,
-				(socklen_t) sizeof(timeout))) {
-		close(sockfd);
-		return 0;
-	}
-	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, (void*)&timeout,
-				(socklen_t) sizeof(timeout))) {
-		close(sockfd);
-		return 0;
+	/* perform nonblocking connect, to be able to wait with select() */
+	ldns_sock_nonblock(sockfd);
+	if (connect(sockfd, (struct sockaddr*)to, tolen) == -1) {
+#ifndef USE_WINSOCK
+#ifdef EINPROGRESS
+		if(errno != EINPROGRESS) {
+#else
+		if(1) {
+#endif
+			close(sockfd);
+			return 0;
+		}
+#else /* USE_WINSOCK */
+		if(WSAGetLastError() != WSAEINPROGRESS &&
+			WSAGetLastError() != WSAEWOULDBLOCK) {
+			closesocket(sockfd);
+			return 0;
+		}
+#endif
+		/* error was only telling us that it would block */
 	}
 
-	if (connect(sockfd, (struct sockaddr*)to, tolen) == -1) {
- 		close(sockfd);
-		return 0;
+	/* wait(write) until connected or error */
+	while(1) {
+		int error = 0;
+		socklen_t len = (socklen_t)sizeof(error);
+
+		if(!ldns_sock_wait(sockfd, timeout, 1)) {
+			close(sockfd);
+			return 0;
+		}
+
+		/* check if there is a pending error for nonblocking connect */
+		if(getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (void*)&error,
+			&len) < 0) {
+#ifndef USE_WINSOCK
+			error = errno; /* on solaris errno is error */
+#else
+			error = WSAGetLastError();
+#endif
+		}
+#ifndef USE_WINSOCK
+#if defined(EINPROGRESS) && defined(EWOULDBLOCK)
+		if(error == EINPROGRESS || error == EWOULDBLOCK)
+			continue; /* try again */
+#endif
+		else if(error != 0) {
+			close(sockfd);
+			/* error in errno for our user */
+			errno = error;
+			return 0;
+		}
+#else /* USE_WINSOCK */
+		if(error == WSAEINPROGRESS)
+			continue;
+		else if(error == WSAEWOULDBLOCK)
+			continue;
+		else if(error != 0) {
+			closesocket(sockfd);
+			errno = error;
+			return 0;
+		}
+#endif /* USE_WINSOCK */
+		/* connected */
+		break;
 	}
 
+	/* set the socket blocking again */
+	ldns_sock_block(sockfd);
+
 	return sockfd;
 }
 
@@ -384,6 +497,58 @@ ldns_udp_read_wire(int sockfd, size_t *size, struct sockaddr_storage *from,
 	return wire;
 }
 
+uint8_t *
+ldns_tcp_read_wire_timeout(int sockfd, size_t *size, struct timeval timeout)
+{
+	uint8_t *wire;
+	uint16_t wire_size;
+	ssize_t bytes = 0;
+
+	wire = LDNS_XMALLOC(uint8_t, 2);
+	if (!wire) {
+		*size = 0;
+		return NULL;
+	}
+	
+	while (bytes < 2) {
+		if(!ldns_sock_wait(sockfd, timeout, 0)) {
+			*size = 0;
+			LDNS_FREE(wire);
+			return NULL;
+		}
+		bytes = recv(sockfd, (void*)wire, 2, 0);
+		if (bytes == -1 || bytes == 0) {
+			*size = 0;
+			LDNS_FREE(wire);
+			return NULL;
+		}
+	}
+
+	wire_size = ldns_read_uint16(wire);
+	
+	LDNS_FREE(wire);
+	wire = LDNS_XMALLOC(uint8_t, wire_size);
+	bytes = 0;
+
+	while (bytes < (ssize_t) wire_size) {
+		if(!ldns_sock_wait(sockfd, timeout, 0)) {
+			*size = 0;
+			LDNS_FREE(wire);
+			return NULL;
+		}
+		bytes += recv(sockfd, (void*) (wire + bytes), 
+				(size_t) (wire_size - bytes), 0);
+		if (bytes == -1 || bytes == 0) {
+			LDNS_FREE(wire);
+			*size = 0;
+			return NULL;
+		}
+	}
+	
+	*size = (size_t) bytes;
+	return wire;
+}
+
 uint8_t *
 ldns_tcp_read_wire(int sockfd, size_t *size)
 {
@@ -441,8 +606,8 @@ ldns_tcp_send(uint8_t **result,  ldns_buffer *qbin, const struct sockaddr_storag
 	if (sockfd == 0) {
 		return LDNS_STATUS_ERR;
 	}
-	
-	answer = ldns_tcp_read_wire(sockfd, answer_size);
+
+	answer = ldns_tcp_read_wire_timeout(sockfd, answer_size, timeout);
 	close(sockfd);
 
 	if (*answer_size == 0) {
@@ -487,6 +652,8 @@ ldns_rdf2native_sockaddr_storage(const ldns_rdf *rd, uint16_t port, size_t *size
         if (!data) {
                 return NULL;
         }
+		/* zero the structure for portability */
+		memset(data, 0, sizeof(struct sockaddr_storage));
         if (port == 0) {
                 port =  LDNS_PORT;
         }
