@@ -456,17 +456,12 @@ ldns_pkt_set_random_id(ldns_pkt *packet)
 {
 	uint16_t rid = 0;
 #ifdef HAVE_SSL
-	unsigned char *rb;
-	rb = LDNS_XMALLOC(unsigned char, 2);
-	if (RAND_bytes(rb, 2) == 1) {
-		rid = ldns_read_uint16(rb);
-	}
-	LDNS_FREE(rb);
-#endif
-	if (rid == 0) {
+	if (RAND_bytes((unsigned char*)&rid, 2) != 1) {
 		rid = (uint16_t) random();
 	}
-
+#else
+	rid = (uint16_t) random();
+#endif
 	ldns_pkt_set_id(packet, rid);
 }
 
@@ -793,6 +788,7 @@ ldns_pkt_free(ldns_pkt *packet)
 		ldns_rr_list_deep_free(packet->_authority);
 		ldns_rr_list_deep_free(packet->_additional);
 		ldns_rr_free(packet->_tsig_rr);
+		ldns_rdf_deep_free(packet->_edns_data);
 		LDNS_FREE(packet);
 	}
 }
@@ -860,6 +856,7 @@ ldns_pkt_query_new_frm_str(ldns_pkt **p, const char *name, ldns_rr_type rr_type,
 		ldns_rr_set_owner(question_rr, name_rdf);
 		ldns_rr_set_type(question_rr, rr_type);
 		ldns_rr_set_class(question_rr, rr_class);
+                ldns_rr_set_question(question_rr, true);
 		
 		ldns_pkt_push_rr(packet, LDNS_SECTION_QUESTION, question_rr);
 	} else {
@@ -910,6 +907,7 @@ ldns_pkt_query_new(ldns_rdf *rr_name, ldns_rr_type rr_type, ldns_rr_class rr_cla
 	ldns_rr_set_owner(question_rr, rr_name);
 	ldns_rr_set_type(question_rr, rr_type);
 	ldns_rr_set_class(question_rr, rr_class);
+        ldns_rr_set_question(question_rr, true);
 	
 	packet->_tsig_rr = NULL;
 	
