@@ -127,13 +127,13 @@ static const EVP_MD *
 ldns_digest_function(char *name)
 {
 	/* these are the mandatory algorithms from RFC4635 */
-	/* The optional algorithms are not yet implemented */	
+	/* The optional algorithms are not yet implemented */
 	if (strlen(name) == 12 && strncasecmp(name, "hmac-sha256.", 11) == 0) {
-#ifdef SHA256_DIGEST_LENGTH
+#ifdef HAVE_EVP_SHA256
 		return EVP_sha256();
 #else
-		return NULL;	
-#endif		
+		return NULL;
+#endif
 	} else if (strlen(name) == 10 && strncasecmp(name, "hmac-sha1.", 9) == 0)
 		return EVP_sha1();
 	else if (strlen(name) == 25 && strncasecmp(name,
@@ -145,10 +145,10 @@ ldns_digest_function(char *name)
 #endif
 
 #ifdef HAVE_SSL
-static ldns_status 
+static ldns_status
 ldns_tsig_mac_new(ldns_rdf **tsig_mac, uint8_t *pkt_wire, size_t pkt_wire_size,
-		const char *key_data, ldns_rdf *key_name_rdf, ldns_rdf *fudge_rdf, 
-		ldns_rdf *algorithm_rdf, ldns_rdf *time_signed_rdf, ldns_rdf *error_rdf, 
+		const char *key_data, ldns_rdf *key_name_rdf, ldns_rdf *fudge_rdf,
+		ldns_rdf *algorithm_rdf, ldns_rdf *time_signed_rdf, ldns_rdf *error_rdf,
 		ldns_rdf *other_data_rdf, ldns_rdf *orig_mac_rdf)
 {
 	char *wireformat;
@@ -161,8 +161,8 @@ ldns_tsig_mac_new(ldns_rdf **tsig_mac, uint8_t *pkt_wire, size_t pkt_wire_size,
 	unsigned int md_len = EVP_MAX_MD_SIZE;
 	ldns_rdf *result = NULL;
 	ldns_buffer *data_buffer = NULL;
-	
-	/* 
+
+	/*
 	 * prepare the digestable information
 	 */
 	data_buffer = ldns_buffer_new(LDNS_MAX_PACKETLEN);
@@ -179,14 +179,14 @@ ldns_tsig_mac_new(ldns_rdf **tsig_mac, uint8_t *pkt_wire, size_t pkt_wire_size,
 	(void)ldns_rdf2buffer_wire(data_buffer, fudge_rdf);
 	(void)ldns_rdf2buffer_wire(data_buffer, error_rdf);
 	(void)ldns_rdf2buffer_wire(data_buffer, other_data_rdf);
-	
+
 	wireformat = (char *) data_buffer->_data;
 	wiresize = (int) ldns_buffer_position(data_buffer);
-	
+
 	algorithm_name = ldns_rdf2str(algorithm_rdf);
-	
+
 	/* prepare the key */
-	key_bytes = LDNS_XMALLOC(unsigned char, 
+	key_bytes = LDNS_XMALLOC(unsigned char,
 			ldns_b64_pton_calculate_size(strlen(key_data)));
 	key_size = ldns_b64_pton(key_data, key_bytes, strlen(key_data) * 2);
 	if (key_size < 0) {
@@ -197,27 +197,27 @@ ldns_tsig_mac_new(ldns_rdf **tsig_mac, uint8_t *pkt_wire, size_t pkt_wire_size,
 	/* 2 spare bytes for the length */
 	mac_bytes = LDNS_XMALLOC(unsigned char, md_len);
 	memset(mac_bytes, 0, md_len);
-	
+
 	digester = ldns_digest_function(algorithm_name);
-	
+
 	if (digester) {
-		(void) HMAC(digester, key_bytes, key_size, (void *)wireformat, wiresize, 
-			    mac_bytes + 2, &md_len);
-	
+		(void) HMAC(digester, key_bytes, key_size, (void *)wireformat,
+		            (size_t) wiresize, mac_bytes + 2, &md_len);
+
 		ldns_write_uint16(mac_bytes, md_len);
-		result = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_INT16_DATA, md_len + 2, 
+		result = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_INT16_DATA, md_len + 2,
 				mac_bytes);
 	} else {
 		return LDNS_STATUS_CRYPTO_UNKNOWN_ALGO;
 	}
-	
+
 	LDNS_FREE(algorithm_name);
 	LDNS_FREE(mac_bytes);
 	LDNS_FREE(key_bytes);
 	ldns_buffer_free(data_buffer);
 
 	*tsig_mac = result;
-	
+
 	return LDNS_STATUS_OK;
 }
 #endif /*  HAVE_SSL */
@@ -225,7 +225,7 @@ ldns_tsig_mac_new(ldns_rdf **tsig_mac, uint8_t *pkt_wire, size_t pkt_wire_size,
 
 #ifdef HAVE_SSL
 bool
-ldns_pkt_tsig_verify(ldns_pkt *pkt, uint8_t *wire, size_t wirelen, 
+ldns_pkt_tsig_verify(ldns_pkt *pkt, uint8_t *wire, size_t wirelen,
 		const char *key_name, const char *key_data, ldns_rdf *orig_mac_rdf)
 {
 	ldns_rdf *fudge_rdf;
@@ -239,12 +239,12 @@ ldns_pkt_tsig_verify(ldns_pkt *pkt, uint8_t *wire, size_t wirelen,
 	ldns_rdf *key_name_rdf = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, key_name);
 	uint16_t pkt_id, orig_pkt_id;
 	ldns_status status;
-	
+
 	uint8_t *prepared_wire = NULL;
 	size_t prepared_wire_size = 0;
-	
+
 	ldns_rr *orig_tsig = ldns_pkt_tsig(pkt);
-	
+
 	if (!orig_tsig) {
 		ldns_rdf_deep_free(key_name_rdf);
 		return false;
@@ -256,7 +256,7 @@ ldns_pkt_tsig_verify(ldns_pkt *pkt, uint8_t *wire, size_t wirelen,
 	orig_id_rdf = ldns_rr_rdf(orig_tsig, 4);
 	error_rdf = ldns_rr_rdf(orig_tsig, 5);
 	other_data_rdf = ldns_rr_rdf(orig_tsig, 6);
-	
+
 	/* remove temporarily */
 	ldns_pkt_set_tsig(pkt, NULL);
 	/* temporarily change the id to the original id */
@@ -265,13 +265,13 @@ ldns_pkt_tsig_verify(ldns_pkt *pkt, uint8_t *wire, size_t wirelen,
 	ldns_pkt_set_id(pkt, orig_pkt_id);
 
 	prepared_wire = ldns_tsig_prepare_pkt_wire(wire, wirelen, &prepared_wire_size);
-	
+
 	status = ldns_tsig_mac_new(&my_mac_rdf, prepared_wire, prepared_wire_size,
-			key_data, key_name_rdf, fudge_rdf, algorithm_rdf, 
+			key_data, key_name_rdf, fudge_rdf, algorithm_rdf,
 			time_signed_rdf, error_rdf, other_data_rdf, orig_mac_rdf);
-	
+
 	LDNS_FREE(prepared_wire);
-	
+
 	if (status != LDNS_STATUS_OK) {
 		ldns_rdf_deep_free(key_name_rdf);
 		return false;
@@ -279,9 +279,9 @@ ldns_pkt_tsig_verify(ldns_pkt *pkt, uint8_t *wire, size_t wirelen,
 	/* Put back the values */
 	ldns_pkt_set_tsig(pkt, orig_tsig);
 	ldns_pkt_set_id(pkt, pkt_id);
-	
+
 	ldns_rdf_deep_free(key_name_rdf);
-	
+
 	if (ldns_rdf_compare(pkt_mac_rdf, my_mac_rdf) == 0) {
 		ldns_rdf_deep_free(my_mac_rdf);
 		return true;
@@ -295,7 +295,7 @@ ldns_pkt_tsig_verify(ldns_pkt *pkt, uint8_t *wire, size_t wirelen,
 #ifdef HAVE_SSL
 /* TODO: memory :p */
 ldns_status
-ldns_pkt_tsig_sign(ldns_pkt *pkt, const char *key_name, const char *key_data, 
+ldns_pkt_tsig_sign(ldns_pkt *pkt, const char *key_name, const char *key_data,
 		uint16_t fudge, const char *algorithm_name, ldns_rdf *query_mac)
 {
 	ldns_rr *tsig_rr;
@@ -306,23 +306,23 @@ ldns_pkt_tsig_sign(ldns_pkt *pkt, const char *key_name, const char *key_data,
 	ldns_rdf *error_rdf = NULL;
 	ldns_rdf *mac_rdf = NULL;
 	ldns_rdf *other_data_rdf = NULL;
-	
+
 	ldns_status status = LDNS_STATUS_OK;
-	
+
 	uint8_t *pkt_wire = NULL;
 	size_t pkt_wire_len;
-	
+
 	struct timeval tv_time_signed;
 	uint8_t *time_signed = NULL;
 	ldns_rdf *time_signed_rdf = NULL;
-	
+
 	algorithm_rdf = ldns_rdf_new_frm_str(LDNS_RDF_TYPE_DNAME, algorithm_name);
 
 	/* eww don't have create tsigtime rdf yet :( */
 	/* bleh :p */
 	if (gettimeofday(&tv_time_signed, NULL) == 0) {
 		time_signed = LDNS_XMALLOC(uint8_t, 6);
-		ldns_write_uint64_as_uint48(time_signed, 
+		ldns_write_uint64_as_uint48(time_signed,
 				(uint64_t)tv_time_signed.tv_sec);
 	} else {
 		status = LDNS_STATUS_INTERNAL_ERR;
@@ -330,7 +330,7 @@ ldns_pkt_tsig_sign(ldns_pkt *pkt, const char *key_name, const char *key_data,
 	}
 
 	time_signed_rdf = ldns_rdf_new(LDNS_RDF_TYPE_TSIGTIME, 6, time_signed);
-	
+
 	fudge_rdf = ldns_native2rdf_int16(LDNS_RDF_TYPE_INT16, fudge);
 
 	orig_id_rdf = ldns_native2rdf_int16(LDNS_RDF_TYPE_INT16, ldns_pkt_id(pkt));
@@ -347,20 +347,20 @@ ldns_pkt_tsig_sign(ldns_pkt *pkt, const char *key_name, const char *key_data,
 	status = ldns_tsig_mac_new(&mac_rdf, pkt_wire, pkt_wire_len,
 			key_data, key_name_rdf, fudge_rdf, algorithm_rdf,
 			time_signed_rdf, error_rdf, other_data_rdf, query_mac);
-	
+
 	if (!mac_rdf) {
 		goto clean;
 	}
-	
+
 	LDNS_FREE(pkt_wire);
-	
+
 	/* Create the TSIG RR */
 	tsig_rr = ldns_rr_new();
 	ldns_rr_set_owner(tsig_rr, key_name_rdf);
 	ldns_rr_set_class(tsig_rr, LDNS_RR_CLASS_ANY);
 	ldns_rr_set_type(tsig_rr, LDNS_RR_TYPE_TSIG);
 	ldns_rr_set_ttl(tsig_rr, 0);
-	
+
 	ldns_rr_push_rdf(tsig_rr, algorithm_rdf);
 	ldns_rr_push_rdf(tsig_rr, time_signed_rdf);
 	ldns_rr_push_rdf(tsig_rr, fudge_rdf);
@@ -368,7 +368,7 @@ ldns_pkt_tsig_sign(ldns_pkt *pkt, const char *key_name, const char *key_data,
 	ldns_rr_push_rdf(tsig_rr, orig_id_rdf);
 	ldns_rr_push_rdf(tsig_rr, error_rdf);
 	ldns_rr_push_rdf(tsig_rr, other_data_rdf);
-	
+
 	ldns_pkt_set_tsig(pkt, tsig_rr);
 
 	return status;
